DisclosureQuestions = [
    'By submitting the manuscript to the Research Transparency Index (RTI), I acknowledge that RTI reserves the right to verify the accuracy of the information I have provided. I agree that if requested, I will supply the data to journal staff to verify the results.',
    'I confirm that I have read and understood the definitions of and requirements concerning plagiarism as defined by The Leadership Quarterly author guidelines (<a href="https://www.elsevier.com/journals/the-leadership-quarterly/1048-9843?generatepdf=true">https://www.elsevier.com/journals/the-leadership-quarterly/1048-9843?generatepdf=true</a>) and the Elsevier ethical guidelines (<a href="https://www.elsevier.com/about/policies/publishing-ethics#Authors">https://www.elsevier.com/about/policies/publishing-ethics#Authors</a>), and I acknowledge that I have not committed any form of plagiarism when writing the paper.',
    "I confirm that the research meets the Elsevier ethical guidelines and that I have obtained Institutional Review Board human subjects' approval where appropriate.",
    "I confirm that all my co-authors have verified the data's integrity and the results' accuracy.",
    "If the manuscript is accepted or has been accepted for publication, I agree that this manuscript's RTI score can be displayed publicly."
]

LevelColor = ['#C00000', '#C45911', '#2E74B5', '#538135']

LevelBackgroundColor = ['#D42F2F', '#FF9226', '#FFD859', '#3EC73E']

SectionDict = {
    'Theory': ['ResearchStrategy', 'PostHocAnalysis'],
    'Research Design': ['TypeofResearchDesign', 'DataCollectionProcedure', 'CountryofDataCollection', 'SamplingMethod'],
    'Measurement': ['AlteredItems', 'ReversedItems', 'MeasurementVariability', 'RangeRestriction'],
    'Data Analysis': ['AnalyticMethod', 'Software', 'Outliers'],
    'Reporting Style': ['Sensitivity', 'EffectSize', 'ResponseRate', 'PValues', 'Coefficients']
}

TransparencyCheckDict = {
    'ResearchStrategy': 'Does the manuscript report the research strategy (e.g., explicitly indicate inductive, deductive, or abductive research)?',
    'PostHocAnalysis': 'Does the manuscript report both supported and unsupported hypotheses for post-hoc hypotheses with detailed explanations?',
    'TypeofResearchDesign': 'Does the manuscript describe the type of research design (e.g., passive observation, experimental)?',
    'DataCollectionProcedure': 'Does the manuscript report the data collection procedure (e.g., surveys, interviews)?',
    'CountryofDataCollection': 'Does the manuscript report the location of data collection (e.g., countries)?',
    'SamplingMethod': 'Does the manuscript report the sampling method (e.g., purposeful, snowball, convenience)?',
    'AlteredItems': 'Does the manuscript report the scale alteration (e.g., dropped items, changes in item referent)?',
    'ReversedItems': 'Does the manuscript report the exact items used in every reversed scale?',
    'MeasurementVariability': 'Does the manuscript report the measurement variability within and between units of analysis (e.g., rwg(j) and ICC)?',
    'RangeRestriction': 'Does the manuscript report the type of range restriction, and provide details regarding the rationale for the decision to correct or not correct?',
    'AnalyticMethod': 'Does the manuscript report the specific analytical method used and why it was chosen (e.g., EFA vs. CFA)?',
    'Software': 'Does the manuscript report the name and version of software used?',
    'Outliers': 'Does the manuscript report the details such as methods and decision rules used to identify outliers and anomalies, if tests for outliers, anomalies, or other inconsistencies were conducted?',
    'Sensitivity': 'Does the manuscript report the results of missing-data analysis (e.g., sensitivity analysis); method (e.g., imputation, deletion) used to address missing data; information (even if speculative) as to why missing data occurred?',
    'ResponseRate': 'Does the manuscript report the exact response rate?',
    'PValues': 'Does the manuscript report exact p-values to two decimal places; do not report p-values compared to cut-offs (e.g., p < .05 or p < .01)?',
    'Coefficients': 'Does the manuscript report both unstandardized and standardized coefficients?'
}

SectionLengthDict = {
    'Theory': 0,
    'Research Design': 0,
    'Measurement': -2,
    'Data Analysis': 1,
    'Reporting Style': 1
}

SectionFullScoreDict = {
    'Theory': 6,
    'Research Design': 8,
    'Measurement': 12,
    'Data Analysis': 10,
    'Reporting Style': 16
}

FullScore1List = [
    'ReversedItems', 'MeasurementVariability'
]

SectionLevelDict = {
    'Theory': [[0], [1, 2], [3], [4]],
    'Research Design': [[0], [1, 2], [3], [4]],
    'Measurement': [[0], [1, 2, 3], [4, 5], [6]],
    'Data Analysis': [[0], [1, 2], [3, 4], [5]],
    'Reporting Style': [[0], [1, 2, 3], [4, 5], [6]]
}



class Report:
    GeneralFeedback = []
    GeneralReplyDict = {}
    GeneralFeedbackScore = [25, 35, 45, 54, 100]
    TitleSuffix = ''
    Conclusion = ''

    def __init__(self, feedback_file=None):
        self._feedback_file = feedback_file
        self.DetailReplyDict = {}
        self.load_detail_reply_dict()

    @staticmethod
    def get_general_feedback_index(score):
        score = score * 2
        for idx, threshold in enumerate(Report.GeneralFeedbackScore):
            if score < threshold:
                return idx

    def load_detail_reply_dict(self):
        if self._feedback_file is None:
            return
        f = open(self._feedback_file, encoding='utf-8')
        line = f.readline()
        while line:
            fb_key = line.strip()
            self.DetailReplyDict[fb_key] = []
            for i in range(3):
                line = f.readline()
                line = line.strip()
                self.DetailReplyDict[fb_key].append(line)
            line = f.readline()

    def generate_result_html(self, score):
        result = f'''<body bgcolor="#D9E2F3" background="image012.gif"><h1 style="text-align: center; font-family:Calibri"><strong>Transparency Report {self.TitleSuffix}</strong></h1>
    <p style="font-family:Calibri; font-size:14px">&nbsp; &nbsp; &nbsp; &nbsp;Thank you for submitting your manuscript. Please see the transparency report below. </p>'''
        if self.GeneralFeedback:
            sum_score = sum([score[x] for x in score])
            general_feedback_index = Report.get_general_feedback_index(sum_score)
            result += '<p style="font-family:Calibri; font-size:14px">&nbsp; &nbsp; &nbsp; &nbsp;{}</p>'.format(
                self.GeneralFeedback[general_feedback_index].format(sum_score * 2 / 52 * 100))
        for section in SectionDict:
            section_score = sum([score[x] for x in SectionDict[section]])
            for idx in range(4):
                if section_score in SectionLevelDict[section][idx]:
                    level = idx
                    break
            result += '<br/><img src="check.jpg">'
            result += '<span style="text-decoration:underline; color:#36597D; font-size:15pt"><strong>{} </strong></span><span style="text-decoration:underline; color:{}; font-size:15pt;"><strong>(LEVEL {})</strong></span>'.format(
                section, LevelColor[level], level)
            result += '<br/><span style=" color:{}; font-size:17pt; margin-left:55px;">&nbsp; &nbsp; &nbsp; &nbsp; {}%</span>\n'.format(LevelColor[level], round(section_score * 2 / SectionFullScoreDict[section] * 100))
            result += '<p style="font-family:Calibri; font-size:14px; color:#36597D">&nbsp; &nbsp; &nbsp; &nbsp;{}</p>\n'.format(
                self.GeneralReplyDict[section][level])
            result += '<ul style="font-family:Calibri; font-size:14px">\n'
            for item in SectionDict[section]:
                print(score)
                if score[item] < 2 and self.DetailReplyDict and self.DetailReplyDict[item][score[item]]:
                    reply = self.DetailReplyDict[item][score[item]]
                    quote = ''
                    if '        ' in reply:
                        reply, quote = reply.split('        ')
                    result += '<li>{}</li>\n'.format(reply)
                    if quote:
                        result += '<table style="font-family:Calibri; font-size:14px; font-style:italic; border-style:dashed; border-width:1px; border-color:#8597b1; border-collapse:collapse;"><tr><td>{}</td></tr></table>\n'.format(quote)
            result += '</ul>'

        result += '<p style="font-family:Calibri; font-size:14px; font-weight: bold; color:#36597D">&nbsp; &nbsp; &nbsp; &nbsp;{}</p></body>' \
            .format(self.Conclusion)

        return result


class AuthorReport(Report):
    GeneralReplyDict = {
        'Theory': [
            'The theory part does not seem to be transparent. We suggest that you revise your manuscript before resubmitting it. Here are our suggestions:',
            'The theory part does not seem to be transparent enough. Please find our suggestions below:',
            'The theory part is broadly transparent. Here are several suggestions for you to improve even further:',
            'The theory part is highly transparent. '
        ],
        'Research Design': [
            'The research design part does not seem to be transparent. We suggest that you revise your manuscript before resubmitting it. Here are our suggestions:',
            'The research design does not seem to be transparent enough. Please find our suggestions below:',
            'The research design is broadly transparent. Here are several suggestions for you to improve further:',
            'The research design is highly transparent.'
        ],
        'Measurement': [
            'The measurement part does not seem to be transparent. We suggest that you revise your manuscript before resubmitting it. Here are our suggestions:',
            'The measurement section does not seem to be transparent enough. Please find our suggestions below:',
            'The measurement section is broadly transparent. Here are several suggestions for you to improve even further:',
            'The measurement section is highly transparent.'
        ],
        'Data Analysis': [
            'The data analysis part does not seem to be transparent. We suggest that you revise your manuscript before resubmitting it. Here are our suggestions:',
            'The data analysis part does not seem to be transparent enough. Please find our suggestions below:',
            'The data analysis part is broadly transparent. Here are several suggestions for you to improve even further:',
            'The data analysis part is highly transparent.'],
        'Reporting Style': [
            'The reporting style does not seem to be transparent. We suggest that you revise your manuscript before resubmitting it. Here are our suggestions:',
            'The reporting does not seem to be transparent enough. Please find our suggestions below:',
            'The reporting seems to be broadly transparent. Here are several suggestions for you to improve even further:',
            'The reporting seems to be highly transparent.'
        ]
    }
    Conclusion = 'We hope that our suggestions can help you enhance your manuscript’s transparency.'

    def __init__(self):
        super().__init__('dict/feedback')


class ReviewerReport(Report):
    TitleSuffix = 'for Reviewers'
    GeneralFeedback = [
        'The manuscript\'s RTI score is {:.2f}%. This indicates that the manuscript has very low transparency and the manuscript would benefit greatly from reporting more research details. Transparency could be a major issue in this paper. We highly recommend that editors ask the authors to provide research details before proceeding to the peer review and that reviewers highlight areas where more transparency is needed in their reviews.  Please find our summary below.',
        'The manuscript\'s RTI score is {:.2f}%. This indicates that the manuscript has low transparency and the manuscript would benefit to some degree from reporting more research details. Transparency may be a major issue in this paper. We thus recommend that editors ask the authors to provide research details before proceeding to the peer review. We also recommend that reviewers highlight areas where more transparency is needed in their reviews. Please find our summary below.',
        'The manuscript\'s RTI score is {:.2f}%. This indicates that the manuscript has moderate transparency and may benefit to some degree from further reporting of research details.  Transparency may not be a major issue in this paper. We thus recommend that editors proceed to peer review or ask the authors to provide research details before proceeding to peer review. We also recommend that reviewers highlight areas where more transparency is needed in their reviews. Please find our summary below.',
        'The manuscript\'s RTI score is {:.2f}%. This indicates that the manuscript is transparent. Transparency may be a minor issue in this paper. We thus recommend that editors proceed directly to peer review. Reviewers may also highlight areas where more transparency is needed in their reviews. Please find our summary below.',
        'The manuscript\'s RTI score is {:.2f}%, which indicates that the manuscript has high transparency. That is, the manuscript has included sufficient information regarding reporting theory, research design, measurement, data analysis, and reporting style. We thus believe that editors can directly proceed to peer review. To be noted, full score does not mean the manuscript is 100% transparent. Reviewers may still need to pay attention to research details in the manuscript and highlight whatever is missing in their reviews.'
    ]
    GeneralReplyDict = {
        'Theory': [
            'The manuscript may greatly benefit from reporting research details regarding theory (i.e., the paper’s goal and the research approach). Here are the detailed feedback:',
            'The manuscript may somewhat benefit from reporting research details regarding theory (i.e., the paper’s goal and the research approach). Please find the detailed information below:',
            'The manuscript is broadly transparent in reporting the theory (i.e., the paper’s goal and the research approach). Here are possible ways for the author to improve:',
            'The theory part (i.e., the paper’s goal and the research approach) seems highly transparent.'
        ],
        'Research Design': [
            'The manuscript may greatly benefit from reporting research details regarding research design. For example, how the study was designed, where the data was collected, and how the data was sampled. Here are the detailed feedback:',
            'The manuscript may benefit to some degree from providing more research details regarding research design. For example, how the study was designed, where the data was collected, and how the data was sampled. Please find the detailed information below:',
            'The manuscript is broadly transparent in reporting the research design. For example, how the study was designed, where the data was collected, and how the data was sampled. Here are possible ways for the author to improve:',
            'The research design (i.e., how the study was designed, where the data was collected, and how the data was sampled) seems highly transparent.'
        ],
        'Measurement': [
            'The manuscript may benefit a great deal from reporting research details regarding measurement. For example, how the scales were developed and whether the measurement was reliable. Here are the detailed feedback:',
            'The manuscript may benefit to some degree from providing more research details. For example, how the scales were developed and whether the measurement was reliable. Please find the detailed information below:',
            'The manuscript is broadly transparent in reporting the measurement. For example, how the scales were developed and whether the measurement was reliable. Here are possible ways for the author to improve:',
            'The measurement part (i.e., how the scales were developed and whether the measurement was reliable) seems highly transparent. '
        ],
        'Data Analysis': [
            'The manuscript may greatly benefit from reporting research details regarding data analysis. For example, information regarding the analytic method and other analysis procedures such as outlier detection. Here are the detailed feedback:',
            'The manuscript may benefit to some degree from research details regarding the data analysis. For example, information regarding the analytic method and other analysis procedures such as outlier detection. Please find the detailed information below:',
            'The manuscript is broadly transparent in reporting the data analysis. For example, information regarding the analytic method and other analysis procedures such as outlier detection. Here are possible ways for the manuscript to improve:',
            'The data analysis part (i.e., information regarding the analytic method and other analysis procedures, such as outlier detection) seems highly transparent. '],
        'Reporting Style': [
            'The manuscript may greatly benefit from incorporating a transparent reporting style  (i.e., the way to report important numbers and results). Here are the detailed feedback:',
            'The manuscript may somewhat benefit from incorporating a transparent reporting style  (i.e., the way to report important numbers and results). Please find the detailed information below:',
            'The manuscript is broadly transparent regarding the reporting style (i.e., how to report important numbers and results). Here are possible ways for the author to improve:',
            'The reporting (i.e., results such as response rates, sensitivity analyses, and how the results were reported) seems highly transparent. '
        ]
    }
    Conclusion = 'We hope that our feedback is helpful for the peer process regarding manuscript transparency.'

    def __init__(self):
        super().__init__('dict/feedback_reviewer')


class PolicyMakerReport(Report):
    GeneralFeedback = [
        'The manuscript\'s RTI score is {:.2f}%, which indicates that the manuscript has very low transparency and the manuscript would benefit greatly from reporting more research details. Thus, this manuscript is not a good scientific reference (e.g., for creating policies). Transparent reporting is important in scientific research. Failing to do so may make it difficult to determine the trustworthiness of the results. Please find our summary below:',
        'The manuscript\'s RTI score is {:.2f}%, which indicates that the manuscript has low transparency and the manuscript would benefit to some degree from  reporting more research details. Thus, this manuscript may not be a good scientific reference (e.g., for creating policies). Transparent reporting is important in scientific research. Not doing so may make it difficult to determine the trustworthiness of the results. Please find our summary below:',
        'The manuscript\'s RTI score is {:.2f}%, which indicates that the manuscript has moderate transparency and the manuscript would benefit to some degree from  reporting more research details. Thus, this manuscript may be an acceptable scientific reference (e.g., for creating policies). We also recommend that you explore other studies in supporting your decisions. Please find our summary below:',
        'The manuscript\'s RTI score is {:.2f}%, which indicates that the manuscript is transparent. We believe that this manuscript has included most of the research details and is a good scientific reference (e.g., create policies). However, you may still want to pay attention to the following parts:',
        'The manuscript\'s RTI score is {:.2f}%, which indicates that the manuscript has high transparency. That is, the manuscript has included sufficient information regarding theory, research design, measurement, data analysis, and reporting style. We thus believe this manuscript can be a good scientific reference (e.g., for creating policies).',
    ]
    GeneralReplyDict = {
        'Theory': [
            'The manuscript is not transparent in the theory part. This means that the manuscript did not report details regarding the goal of the manuscript and the research approach. Thus, it would be necessary to refer to more transparent studies (e.g., for creating policies).',
            'The theory part is not sufficiently transparent. This means that the manuscript has only included a limited amount of information regarding the goal of the manuscript and the research approach. Therefore, the theory part should only be relied upon (e.g., in creating policy) in conjunction with one or two other studies.',
            'The theory part is broadly transparent. This means that the manuscript has included an ample amount of information regarding the goal of the manuscript and the research approach.  However, the manuscript can be further improved from reporting more research details regarding theory. Thus, you may still benefit from the study (e.g., creating policies), but it may be helpful to find one or two similar studies.',
            'The theory part has high transparency. This means that the manuscript has included sufficient information regarding the goal of the manuscript and the research approach. Thus, the theory in the manuscript can be relied upon (e.g., for creating policies).'
        ],
        'Research Design': [
            'The manuscript is not transparent in the research design part. This means that the manuscript did not report details regarding how the study was designed, where the data was collected and how the data was sampled. Thus, it might be necessary to refer to more transparent studies (e.g., for creating policies).',
            'The manuscript is not sufficiently transparent in reporting the research design. This means that the manuscript has only included a limited amount of information regarding how the scales were developed, and whether the measurement was reliable. Therefore, the measurement part should only be relied upon (e.g., in creating policy) in conjunction with one or two other studies.',
            'The manuscript is broadly transparent in reporting the research design. This means that the manuscript has included an ample amount of information regarding how the study was designed, where the data was collected and how the data was sampled. However, the manuscript can be further improved from reporting more research details regarding research design. Thus, you may still benefit from the study (e.g., creating policies), but it may be helpful to find one or two similar studies.',
            'The research design part has high transparency. This means that the manuscript has included sufficient information regarding how the study was designed, where the data was collected and how the data was sampled. Thus, the research design in the manuscript can be relied upon (e.g., for creating policies).'
        ],
        'Measurement': [
            'The manuscript is not transparent in the measurement part. This means that the manuscript did not report details regarding how the scales were developed, and whether the measurement was reliable. Thus, it would be necessary for policy makers to refer to more transparent studies (e.g., for creating policies).',
            'The manuscript is not sufficiently transparent in reporting the measurement. This means that the manuscript has only included a limited amount of information regarding how the scales were developed, and whether the measurement was reliable. Therefore, the measurement part should only be relied upon (e.g., in creating policy) in conjunction with one or two other studies.',
            'The measurement part is broadly transparent. This means that the manuscript has included an ample amount of information regarding how the scales were developed, and whether the measurement was reliable.  However, the manuscript can be further improved from reporting more research details regarding measurement. Thus, you may still benefit from the study (e.g., creating policies), but it may be helpful to find one or two similar studies. ',
            'The measurement part has very high transparency. This means that the manuscript has included sufficient information regarding how the scales were developed, and whether the measurement was reliable. Thus, the measurement in the manuscript has high transparency and can be relied upon (e.g., for creating policies).'
        ],
        'Data Analysis': [
            'The manuscript is not transparent in the data analysis part. This means that the manuscript did not report details regarding the analytic method, and other analysis procedures such as outlier detection. Thus, it would be necessary to refer to more transparent studies (e.g., for creating policies).',
            'The manuscript is not sufficiently transparent in reporting the data analysis. This means that the manuscript has only included a limited amount of information regarding the analytic method, and other analysis procedures such as outlier detection. Therefore, the data analysis part should only be relied upon (e.g., in creating policy) in conjunction with one or two other studies.',
            'The data analysis part is broadly transparent. This means that the manuscript has included an ample amount of information regarding the analytic method, and other analysis procedures such as outlier detection.  However, the manuscript can be further improved from reporting more research details regarding data analysis. Thus, you may still benefit from the study (e.g., creating policies), but it may be helpful to find one or two similar studies.',
            'The data analysis part has high transparency. This means that the manuscript has included sufficient information regarding the analytic method, and other analysis procedures such as outlier detection. Thus, the data analysis in the manuscript can be relied upon (e.g., for creating policies).'],
        'Reporting Style': [
            'The manuscript is not transparent in reporting style. This means that the manuscript did not report details about results such as response rates, effect sizes, sensitivity analyses and how the results were reported. Thus, it would be necessary to refer to more transparent studies (e.g., for creating policies).',
            'The manuscript is not sufficiently transparent in general reporting. This means that the manuscript has only included a limited amount of details such as response rates, effect sizes, sensitivity analyses and how the results were reported. Therefore, the reporting should only be relied upon (e.g., in creating policy) in conjunction with one or two other studies.',
            'The reporting is broadly transparent. This means that the manuscript has reported an ample amount of details such as response rates, effect sizes, sensitivity analyses and how the results were reported.  However, the manuscript can be further improved from reporting more research details in the appropriate format. Thus, you may still benefit from the study (e.g., creating policies), but it may be helpful to find one or two similar studies.',
            'The reporting has very high transparency. This means that the manuscript did report details about results such as response rates, effect sizes, sensitivity analyses and how the results were reported. Thus, the reporting in the manuscript can be relied upon (e.g., for creating policies).'
        ]
    }
    Conclusion = 'We hope that our feedback can help you with selecting good scientific references and drawing useful conclusions (e.g., creating policies).'
