﻿
import random
import sys
from functools import partial
import logging

from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtWidgets import (QCheckBox, QFileDialog, QGridLayout, QHBoxLayout,
                             QLabel, QLineEdit, QMessageBox, QTextEdit,
                             QVBoxLayout)

import content
from process import ArticleGrading


logger = logging.getLogger(__name__)

def create_gradient_brush():
    verGradient = QtGui.QLinearGradient(0, 0, 0, 600)
    gradient = verGradient
    gradient.setColorAt(0.0, QtGui.QColor(233, 237, 248))
    gradient.setColorAt(1.0, QtGui.QColor(187, 201, 230))
    brush = QtGui.QBrush(gradient)
    return brush


class ReportDialog(QtWidgets.QDialog):
    def __init__(self, html: str, parent=None):
        super().__init__(parent)
        self._html = html
        self.init_ui()

    def init_ui(self):
        title_label = QLabel(self)
        title_label.setAlignment(QtCore.Qt.AlignCenter)
        pix = QtGui.QPixmap('logo.png')
        title_label.setPixmap(pix)



        self.results = QTextEdit(self)
        self.results.setReadOnly(True)
        self.results.setHtml(self._html)
        v_box = QVBoxLayout()
        self.setLayout(v_box)
        v_box.addWidget(title_label)
        v_box.addWidget(self.results)
        v_box.setContentsMargins(50, 50, 50, 50)
        self.resize(1000, 750)
        self.setWindowTitle('Transparency Report')
        p = QtGui.QPalette()
        p.setBrush(QtGui.QPalette.Window, create_gradient_brush())
        self.setPalette(p)



class DeclarationDialog(QtWidgets.QDialog):
    def __init__(self, declaration_list, transparency_check, parent=None):
        super().__init__(parent)
        self._declaration_list = declaration_list
        self._transparency_check = transparency_check
        self.inputs = []
        self.option_general_a_list = []
        self.option_general_b_list = []
        self.option_a_list = []
        self.option_b_list = []
        self.user_inputs = []
        self.init_ui()
        p = QtGui.QPalette()
        p.setBrush(QtGui.QPalette.Window, create_gradient_brush())
        self.setPalette(p)

    def link(self, linkStr):

        QtGui.QDesktopServices.openUrl(QtCore.QUrl(linkStr))

    def init_ui(self):
        vbox_all = QVBoxLayout()

        logo_label = QLabel(self)
        pix = QtGui.QPixmap('logo.png')
        logo_label.setPixmap(pix)
        logo_label.setAlignment(QtCore.Qt.AlignCenter)

        title_label = QLabel(self)
        title_label.setText("Disclosure Form")
        title_label.setAlignment(QtCore.Qt.AlignCenter)

        title_label.setFont(QtGui.QFont("Helvetica", 15, QtGui.QFont.Bold))

        vbox_all.addSpacing(15)
        vbox_all.addWidget(logo_label)
        vbox_all.addWidget(title_label)
        vbox_all.addSpacing(15)

        scroll_vbox = QVBoxLayout()

        #for t in ['Before we <u>proceed with</u> your paper, please respond to the following questions and statements:',
        #          '<span style="background-color:#E6E6E6"><b>General Information</b></span>']:
        for t in ['Please answer the following questions. If you would like to skip this form, click “Cancel”:',
                  '<span style="background-color:#E6E6E6"><b>General Information</b></span>']:
            sec1_label = QLabel(self)
            sec1_label.setTextFormat(QtCore.Qt.RichText)
            sec1_label.setText(t)
            scroll_vbox.addWidget(sec1_label)
            scroll_vbox.addSpacing(10)

        for idx, x in enumerate(content.DisclosureQuestions):
            title = QLabel(self)
            title.setTextFormat(QtCore.Qt.RichText)
            title.setText(f'<html>{idx+1}. {x}</html>')
            title.setWordWrap(True)
            title.linkActivated.connect(self.link)

            option_a = QCheckBox("a. Yes", self)
            option_a.stateChanged.connect(partial(self.check_event, idx, False))
            option_b = QCheckBox("b. No", self)
            option_b.stateChanged.connect(partial(self.check_event, idx, True))
            self.option_a_list.append(option_b)
            self.option_b_list.append(option_a)

            vbox = QVBoxLayout()
            vbox.addWidget(title)
            for w in [option_a, option_b]:
                input_hbox = QHBoxLayout()
                input_hbox.addSpacing(20)
                input_hbox.addWidget(w)
                vbox.addLayout(input_hbox)
            if idx == 0 or idx == 6:
                declare_input = QLineEdit()
                declare_input.setDisabled(True)
                self.inputs.append(declare_input)
                input_hbox = QHBoxLayout()
                input_hbox.addSpacing(40)
                l = QLabel(self)
                l.setText("• If no, please provide justifications:")
                input_hbox.addWidget(l)
                vbox.addLayout(input_hbox)
                input_hbox = QHBoxLayout()
                input_hbox.addSpacing(40)
                input_hbox.addWidget(declare_input)
                vbox.addLayout(input_hbox)
            else:
                self.inputs.append(None)
            scroll_vbox.addLayout(vbox)

        sec1_label = QLabel(self)
        sec1_label.setTextFormat(QtCore.Qt.RichText)
        sec1_label.setText('<span style="background-color:#E6E6E6"><b>Paper Transparency</b></span>')
        scroll_vbox.addSpacing(10)
        scroll_vbox.addWidget(sec1_label)
        scroll_vbox.addSpacing(10)

        for idx, x in enumerate(self._declaration_list):

            idx += len(content.DisclosureQuestions)
            title = QLabel("{}. {}.".format(idx + 1, x[0]),
                           self)
            option_a = QCheckBox("a. {}".format(x[1]), self)
            option_a.stateChanged.connect(partial(self.check_event, idx, True))
            self.option_a_list.append(option_a)
            option_b = QCheckBox("b. {}".format(x[2]), self)
            option_b.stateChanged.connect(partial(self.check_event, idx, False))
            self.option_b_list.append(option_b)
            declare_input = QLineEdit()
            declare_input.setDisabled(True)
            self.inputs.append(declare_input)

            vbox = QVBoxLayout()
            vbox.addWidget(title)
            for w, s in zip([option_a, declare_input, option_b], [20, 37, 20]):
                input_hbox = QHBoxLayout()
                input_hbox.addSpacing(s)
                input_hbox.addWidget(w)
                vbox.addLayout(input_hbox)

            scroll_vbox.addLayout(vbox)

        sec2_label = QLabel(self)
        sec2_label.setTextFormat(QtCore.Qt.RichText)
        sec2_label.setText('<span style="background-color:#E6E6E6"><b>Transparency Check</b></span>')
        scroll_vbox.addSpacing(10)
        scroll_vbox.addWidget(sec2_label)
        scroll_vbox.addSpacing(10)

        for idx, x in enumerate(self._transparency_check):
            idx += len(content.DisclosureQuestions) + len(self._declaration_list)
            title = QLabel("{}. {}".format(idx + 1, x), self)
            title.setWordWrap(True)
            option_a = QCheckBox("a. Yes", self)
            option_a.stateChanged.connect(partial(self.check_event, idx, True))
            self.option_a_list.append(option_a)
            option_b = QCheckBox("b. No", self)
            option_b.stateChanged.connect(partial(self.check_event, idx, False))
            self.option_b_list.append(option_b)
            self.inputs.append(None)
            vbox = QVBoxLayout()
            vbox.addWidget(title)
            for w in [option_a, option_b]:
                input_hbox = QHBoxLayout()
                input_hbox.addSpacing(20)
                input_hbox.addWidget(w)
                vbox.addLayout(input_hbox)

            scroll_vbox.addLayout(vbox)

        confirm_button = QtWidgets.QPushButton(self)
        confirm_button.setText("Confirm")
        confirm_button.clicked.connect(self.confirm)

        cancel_button = QtWidgets.QPushButton(self)
        cancel_button.setText("Cancel")
        cancel_button.clicked.connect(self.close)

        button_hbox = QHBoxLayout()
        button_hbox.addStretch(1)
        button_hbox.addWidget(confirm_button)
        button_hbox.addWidget(cancel_button)

        scroll = QtWidgets.QScrollArea(self)

        scroll_widget = QtWidgets.QWidget(self)
        scroll_widget.setLayout(scroll_vbox)
        scroll_widget.resize(450, scroll_widget.height())
        scroll_widget.setStyleSheet('background-image:url(image012.gif);')

        scroll.setWidget(scroll_widget)
        scroll.setWidgetResizable(True)
        scroll.setMinimumHeight(450)
        scroll.setMaximumHeight(600)
        scroll.setMinimumWidth(600)
        vbox_all.addWidget(scroll)

        vbox_all.addLayout(button_hbox)
        vbox_all.addSpacing(15)
        self.setLayout(vbox_all)

        self.setWindowTitle('Disclosure Form')

        self.resize(1000, self.height())

    def check_event(self, idx, is_option_a):
        if is_option_a:
            if self.option_a_list[idx].isChecked():
                if self.inputs[idx] is not None:
                    self.inputs[idx].setDisabled(False)
                self.option_b_list[idx].setChecked(False)
            elif self.inputs[idx] is not None:
                self.inputs[idx].setDisabled(True)
        else:
            if self.option_b_list[idx].isChecked():
                self.option_a_list[idx].setChecked(False)

    @QtCore.pyqtSlot()

    def confirm(self):
        for idx in range(len(self.option_a_list)):
            if (not self.option_a_list[idx].isChecked()) and (not self.option_b_list[idx].isChecked()):
                QMessageBox.warning(self,
                                    "Warning",
                                    "Please answer all questions.",
                                    QMessageBox.Yes)
                return
        for idx in range(1, 5):
            if not self.option_b_list[idx].isChecked():
                QMessageBox.warning(self,
                                    "Warning",
                                    "You have indicated “No” for at least one general information question. However, your cooperation is of utmost importance to us. Please check if you have understood and certified our mandatory statements.",
                                    QMessageBox.Yes)
                return
        for idx in range(len(content.DisclosureQuestions), len(self.option_a_list) - len(self._transparency_check)):
            if self.option_a_list[idx].isChecked() and self.inputs[idx].text() == "":
                QMessageBox.warning(self,
                                    "Warning",
                                    "Please input your explanation if you choose to declare.",
                                    QMessageBox.Yes)
                return
        for idx, x in enumerate(self._transparency_check):
            if self.option_a_list[idx + len(content.DisclosureQuestions) + len(self._declaration_list)].isChecked() != self._transparency_check[x]:
                logger.warning(f"User input for [{x}] is not consistent with the manuscript. input: {self.option_a_list[idx + len(content.DisclosureQuestions) + len(self._declaration_list)].isChecked()}, manuscript: {self._transparency_check[x]}")
                transparency_check_content = x[len('Does your manuscript report'): -1]
                if self.option_a_list[idx + len(content.DisclosureQuestions) + len(self._declaration_list)].isChecked():
                    QMessageBox.warning(self,
                                        "Warning",
                                        f"You indicated that you reported {transparency_check_content}, but we did not find relevant information in your manuscript. Please check your manuscript and submit your manuscript again.",
                                        QMessageBox.Yes)
                else:
                    QMessageBox.warning(self,
                                        "Warning",
                                        f"You indicated that you did not report {transparency_check_content}, but we did find relevant information in your manuscript. Please check your manuscript and submit your manuscript again.",
                                        QMessageBox.Yes)
                return
        self.user_inputs = [x.isChecked() for x in self.option_a_list]
        self.user_inputs = self.user_inputs[len(content.DisclosureQuestions):]

        self.accept()
        self.close()


class MyWindow(QtWidgets.QWidget):
    def __init__(self):
        super(MyWindow, self).__init__()
        self.init_ui()
        self.processor = ArticleGrading()

    def init_ui(self):
        self.init_upload_ui()
        p = QtGui.QPalette()
        p.setBrush(QtGui.QPalette.Window, create_gradient_brush())
        self.setPalette(p)

    def init_upload_ui(self):
        self.title_label = QLabel(self)
        self.title_label.setText("Transparency Research Index")
        self.title_label.setAlignment(QtCore.Qt.AlignCenter)

        self.title_label.setFont(QtGui.QFont("Helvetica", 18, QtGui.QFont.Bold))

        pix = QtGui.QPixmap('logo.png')
        self.title_label.setPixmap(pix)

        # self.word_label = QLabel(self)
        # pix_word = QtGui.QPixmap('word.png')
        # pix_word = pix_word.scaled(250, 200, aspectRatioMode=QtCore.Qt.KeepAspectRatio)
        # self.word_label.setPixmap(pix_word)

        self.openButton = QtWidgets.QPushButton(self)
        self.openButton.setObjectName("openButton")
        self.openButton.setText("Submit")
        self.openButton.setFont(QtGui.QFont("Helvetica", 14, QtGui.QFont.Bold))
        self.openButton.setStyleSheet("QPushButton{color:white}"
                                      "QPushButton{background-color:rgb(77,115,190)}"
                                      "QPushButton{border-radius:8px}")
        self.openButton.setMinimumHeight(30)
        self.openButton.setMinimumSize(120, 30)
        self.openButton.clicked.connect(self.msg)

        self.file_path = QLineEdit(self)
        self.file_path.setStyleSheet("QLineEdit{border-radius:8px}")
        self.file_path.setMinimumSize(300, 30)
        self.file_path.setReadOnly(True)

        self.v_box = QVBoxLayout()

        self.setup_upload_ui()

        self.citation = QLabel(self)
        self.citation.setTextFormat(QtCore.Qt.RichText)
        self.citation.setText("To cite this program and for detailed instructions on how to use it, please see the following open-access article:<br>"
                              "Aguinis, H., Li, A. Z., & Foo, M. D. 2024. The research transparency index."
                              "<i>The Leadership Quarterly.</i><br>"
                              "https://doi.org/10.1016/j.leaqua.2024.101809"
                              )
        self.citation.setAlignment(QtCore.Qt.AlignCenter)
        self.citation.setFont(QtGui.QFont("Helvetica", 14))
        self.citation.setStyleSheet("QLabel { color: #4A4A4A; }")
       # self.citation.setWordWrap(True)

        self.v_box.addStretch(1)
        self.v_box.addWidget(self.citation, alignment=QtCore.Qt.AlignCenter)

        self.setLayout(self.v_box)

        self.setWindowTitle('Research Transparency Index')

        self.resize(600, 480)

    def setup_upload_ui(self):
        self.clear_widget()

        self.openButton.show()
        self.file_path.show()
        # self.word_label.show()

        self.file_path.setText('')
        self.file_path.setPlaceholderText('Click the “Submit” button to upload your PDF file')

        upload_h_box = QHBoxLayout()
        upload_h_box.addStretch(1)
        upload_h_box.addWidget(self.file_path)
        upload_h_box.addWidget(self.openButton)
        upload_h_box.addStretch(1)

        self.v_box.addStretch(2)
        self.v_box.addWidget(self.title_label)
        word_hbox = QHBoxLayout()
        word_hbox.addStretch(1)
        # word_hbox.addWidget(self.word_label)
        word_hbox.addStretch(1)
        self.v_box.addStretch(2)
        self.v_box.addLayout(word_hbox)
        self.v_box.addStretch(1)
        self.v_box.addLayout(upload_h_box)
        self.v_box.addStretch(2)


    def msg(self):
        file_name, file_type = QFileDialog.getOpenFileName(self,
                                                           "Open File",
                                                           "./",
                                                           "PDF Files (*.pdf)")
        if file_name:
            self.file_path.setText(file_name)
            
            # ppp = file_name.rfind('/')
            # file_name = file_name[ppp:]
            # file_name = file_name.replace('.docx', '')
            score_dict = self.processor.grade_article(file_name)
            print(score_dict)
            for x in score_dict:
                if score_dict[x] is None:
                    score_dict[x] = 0


            declaration_list = []
            declaration_dict = {}

            if score_dict['PostHocAnalysis'] == 0:
                declaration_list.append(
                    ['Did the manuscript report any post hoc analyses?',
                     'It is not necessary for the manuscript to contain any post-hoc analyses.',
                     'The manuscript did not report necessary post-hoc analyses which have been conducted.'])
                declaration_dict['PostHocAnalysis'] = 0
            if score_dict['AlteredItems'] == 0:
                declaration_list.append(
                    ['Did the manuscript report any adapted items?',
                     'When collecting data, the original items were used, and none were adapted.',
                     'When collecting data, at least one scale was used by adapting from an original scale, but the manuscript did not report detailed information.'])
                declaration_dict['AlteredItems'] = 0
            if score_dict['ReversedItems'] == 0:
                declaration_list.append(
                    ['Did any of the scales used in the manuscript contain reverse-coded items?',
                     'None of the scales used in the manuscript contained reverse-coded items.',
                     'At least one of the scales contained reverse-coded items, but the manuscript did not indicate the items.'])
                declaration_dict['ReversedItems'] = 0
            if score_dict['MeasurementVariability'] == 0:
                declaration_list.append(
                    ['Did any of the scales used in the manuscript contain multi-items?',
                     'None of the scales used in the manuscript contained multi-items.',
                     'At least one of the scales contained multi-items, but the manuscript did not report any inter-rater agreement related information.'])
                declaration_dict['MeasurementVariability'] = 0
            if score_dict['RangeRestriction'] == 0:
                declaration_list.append(
                    ['Did the manuscript report any discussion regarding range restriction?',
                     'Range restriction analysis was not conducted and thus the manuscript does not need to report discussion regarding range restriction.',
                     'No, the manuscript did not report discussion regarding range restriction.'])
                declaration_dict['RangeRestriction1'] = 0
                # declaration_list.append(
                  #  ['Did the manuscript report any discussion regarding measurement error?',
                   #  'Measurement error analysis was not conducted and thus the manuscript does not need to report discussion regarding measurement error.',
                    # 'No, the manuscript did not report discussion regarding measurement error.'])
                # declaration_dict['RangeRestriction2'] = 0
            if score_dict['Outliers'] == 0:
                declaration_list.append(
                    ['Did the manuscript report any discussion regarding outlier analysis?',
                     'Outlier was not found and thus the manuscript does not need to report discussion regarding outliers.',
                     'No, the manuscript did not report discussion regarding outlier analysis.'])
                declaration_dict['Outliers'] = 0
            #if score_dict['Sensitivity'] == 0:
             #   declaration_list.append(
              #3      ['Did the manuscript report any discussion regarding sensitivity analysis?',
                #     'Sensitivity analysis was not conducted and thus the manuscript does not need to report discussion regarding sensitivity analysis.',
                 #    'No, the manuscript did not report discussion regarding sensitivity analysis.'])
                #declaration_dict['Sensitivity'] = 0
            #if score_dict['Preregistration'] == 0:
                declaration_list.append(
                    ['Did the manuscript include any preregistration of the study?',
                     'Preregistration was not required; therefore, the manuscript does not need to report any preregistration.',
                     'No, the manuscript did not report any preregistration of the study.'])
                #declaration_dict['Preregistration'] = 0
            #if score_dict['DataOpenness'] == 0:
                declaration_list.append(
                    ['Did the manuscript report on data openness or on data sharing?',
                     'Data openness was not required; therefore, the manuscript does not need to report on data openness.',
                     'No, the manuscript did not report on data openness or on data sharing.'])
                #declaration_dict['DataOpenness'] = 0 
            #print('process done')

            transparency_check = {}
            for item in content.TransparencyCheckDict:
                if score_dict[item] == 0 and item not in declaration_list:
                    transparency_check[content.TransparencyCheckDict[item]] = False
                elif score_dict[item] == 1 and item in content.FullScore1List:
                    transparency_check[content.TransparencyCheckDict[item]] = True
                elif score_dict[item] == 2:
                    transparency_check[content.TransparencyCheckDict[item]] = True

            if len(transparency_check) > 3:
                transparency_check = {x[0]: x[1] for x in random.sample(sorted(transparency_check.items()), 3)}

            print(transparency_check)



            d = DeclarationDialog(declaration_list, transparency_check, self)
            d.setAttribute(QtCore.Qt.WA_DeleteOnClose)
            if d.exec_() == QtWidgets.QDialog.Accepted:
                RangeRestrictionOptions = []
                for item, declaration in zip(declaration_dict, d.user_inputs):
                    if declaration:
                        if item in score_dict:
                            score_dict[item] = 1 if item in content.FullScore1List else 2
                        else:
                            RangeRestrictionOptions.append(declaration)
                if len(RangeRestrictionOptions) == 2:
                    score_dict['RangeRestriction'] = 1

            # 0809 delete LevelofTheory item

            score_dict['LevelofTheory'] = 0
            self.results_ui(score=score_dict)

        #else:
         #   self.close()
            
    def show_report(self, report_helper=1, score=2):
        r = ReportDialog(report_helper.generate_result_html(score), self)
        r.setAttribute(QtCore.Qt.WA_DeleteOnClose)
        r.exec_()

    def clear_widget(self):
        for cnt in reversed(range(self.v_box.count())):
            widget = self.v_box.takeAt(cnt).widget()

            if widget is not None and widget != self.title_label:
                # widget will be None if the item is a layout
                widget.deleteLater()

    def results_ui(self, score):
        self.openButton.hide()
        self.file_path.hide()
        # self.word_label.hide()

        self.clear_widget()

        self.v_box.addStretch(1)
        self.v_box.addWidget(self.title_label)
        self.v_box.addStretch(1)

        #for target, helper in zip(['Authors and Learners', 'Reviewers and Editors', 'Practitioners'],
         #                         [content.AuthorReport(), content.ReviewerReport(), content.PolicyMakerReport()]):
        for target, helper in zip(['Authors', 'Reviewers/Editors'],
            [content.AuthorReport(), content.ReviewerReport()]):
            report_button = QtWidgets.QPushButton(self)
            report_button.setText(f'Report for {target}')
            report_button.setFont(QtGui.QFont("Helvetica", 14, QtGui.QFont.Bold))
            report_button.setStyleSheet("QPushButton{color:white}"
                                        "QPushButton{background-color:rgb(77,115,190)}"
                                        "QPushButton{border-radius:8px}")
            report_button.setMinimumHeight(60)
            report_button.setMinimumSize(200, 60)
            report_button.setFixedWidth(400)
            report_button.clicked.connect(partial(self.show_report, helper, score))

            self.v_box.addWidget(report_button, alignment=QtCore.Qt.AlignCenter)
            self.v_box.addStretch(1)
        back_button = QtWidgets.QPushButton(self)
        back_button.clicked.connect(self.setup_upload_ui)
        back_button.setText('Back')
        back_button.setFont(QtGui.QFont("Helvetica", 14, QtGui.QFont.Bold))
        back_button.setStyleSheet("QPushButton{color:white}"
                                    "QPushButton{background-color:rgb(77,115,190)}"
                                    "QPushButton{border-radius:8px}")
        back_button.setMinimumSize(80, 40)
        self.v_box.addWidget(back_button, alignment=QtCore.Qt.AlignRight)
        self.v_box.addStretch(1)
        self.v_box.setContentsMargins(50, 50, 50, 50)


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    myshow = MyWindow()
    myshow.show()
    sys.exit(app.exec_())
