import sys
import importlib
importlib.reload(sys)

from pdfminer.pdfparser import PDFParser
from pdfminer.pdfdocument import PDFDocument
from pdfminer.pdfinterp import PDFResourceManager, PDFPageInterpreter
from pdfminer.converter import PDFPageAggregator
from pdfminer.layout import LTTextBoxHorizontal,LAParams
from pdfminer.pdfpage import PDFTextExtractionNotAllowed
from pdfminer.pdfpage import PDFPage

def load_pdf(file_name):
    fp = open(file_name, 'rb')
    parser = PDFParser(fp)
    doc = PDFDocument(parser)
    parser.set_document(doc)
    #doc.set_parser(parser)
    #doc.initialize()
    raw_data = []
    if not doc.is_extractable:
        return None
    else:
        rsrcmgr = PDFResourceManager()
        laparams = LAParams()
        device = PDFPageAggregator(rsrcmgr, laparams=laparams)
        interpreter = PDFPageInterpreter(rsrcmgr, device)

        #for page in doc.get_pages():
        for i, page in enumerate(PDFPage.create_pages(doc)):
            interpreter.process_page(page)
            layout = device.get_result()
            for x in layout:
                if isinstance(x, LTTextBoxHorizontal):
                    with open('res', 'a') as f:
                        results = x.get_text()
                        raw_data.append(results)

    para_data = []
    for line in raw_data:
        line = line.replace('\n', '')
        if len(line) < 3 and line.isalpha():
            continue
        para_data.append(line)
    return para_data


def is_number(str):
    try:
        float(str)
        return True
    except:
        return False

class ArticleGrading():

    def __init__(self):
        self.__country_list = []
        self.__long_method = []
        self.__short_method = []
        self.__software_list = []
        self.__dict_dir = './dict/'
        self.__load_dict()
        self.debug = 0

    def __load_dict(self):
        for line in open(self.__dict_dir + '/country'):
            self.__country_list.append(line.strip().lower())
        for line in open(self.__dict_dir + '/method'):
            if line.strip().find(' ') > 0:
                self.__long_method.append(line.strip())
            else:
                self.__short_method.append(line.strip())
        for line in open(self.__dict_dir + '/software'):
            self.__software_list.append(line.strip().lower())

    def grade_article(self, file_name):
        text_line = self.__load_data(file_name)
        text_line, token_line = self.__split_word(text_line)

        return_score = {}
        return_score['ResearchStrategy'] = self.__research_strategy(text_line, token_line)
        return_score['LevelofTheory'] = self.__level_of_theory(text_line, token_line)
        return_score['PostHocAnalysis'] = self.__post_hoc(text_line, token_line)

        return_score['TypeofResearchDesign'] = self.__research_design(text_line, token_line)
        return_score['DataCollectionProcedure'] = self.__data_collection_procedure(text_line, token_line)
        return_score['CountryofDataCollection'] = self.__country_of_data(text_line, token_line)
        return_score['SamplingMethod'] = self.__sampling_method(text_line, token_line)

        return_score['AlteredItems'] = self.__altered_items(text_line, token_line)
        return_score['ReversedItems'] = self.__reversed_items(text_line, token_line)
        return_score['MeasurementVariability'] = self.__measurement_variability(text_line,token_line)
        return_score['RangeRestriction'] = self.__range_restriction(text_line, token_line)

        return_score['AnalyticMethod'] = self.__analytic_method(text_line, token_line)
        return_score['Software'] = self.__software(text_line, token_line)
        return_score['Outliers'] = self.__outliers(text_line, token_line)
        
        return_score['Sensitivity'] = self.__sensitivity(text_line, token_line)
        return_score['ResponseRate'] = self.__response_rate(text_line, token_line)
        return_score['PValues'] = self.__p_values(text_line, token_line)
        return_score['Coefficients'] = self.__coefficients(text_line, token_line)

        return_score['EffectSize'] = self.__effect_size(text_line, token_line)

        return return_score

    def __load_data(self, file_name):
        #data_line = []
        #for line in open(file_name, encoding='utf8'):
        #    data_line.append(line.strip())
        return load_pdf(file_name)

    def __split_word(self, text_line):
        token_line = []
        new_text_line = []
        for line in text_line:
            lower_text = line.lower()
            token_line.append(lower_text.split(' '))
            new_text_line.append(lower_text)
        return new_text_line, token_line

    def __research_strategy(self, text_line, token_line):
        score = 0
        for text in text_line:
            if text.find('inductive') >= 0 or  \
                text.find('deductive') >= 0 or text.find('abductive') >= 0:
                    score = 0
                    if self.debug == 1:
                        print ('Research strategy:\t' + ' '.join(tokens))
        return score

    def __level_of_theory(self, text_line, token_line):
        score = 0
        introduction = False
        for i in range(len(text_line)):
            line = text_line[i]
            tokens = token_line[i]
            if line.startswith('keywords'):
                introduction = True
            if len(tokens) <= 3:
                if introduction is True:
                    introduction = False
                    break
                for tk in tokens:
                    if tk == 'introduction':
                        introduction = True
            if introduction is False:
                continue
            if line.find('individual level') >= 0 or line.find('team level') >=0 \
                or line.find('firm level') >= 0 or line.find('individual-level') >= 0 \
                or line.find('team-level') >= 0 or line.find('firm-level') >= 0:
                if self.debug == 1:
                    print ('Level of theroy\t' + line.strip())
                    score = 1
        return score

    def __post_hoc(self, text_line, token_line):
        score = 0
        for i in range(len(text_line)):
            line = text_line[i]
            tokens = token_line[i]
            if line.find('post hoc analy') >= 0 or line.find('post-hoc analy') >= 0 \
                or line.find('supplementary analy') >= 0:
                if len(tokens) <= 5: 
                    if self.debug == 1:
                        print ('Post hoc 2:\t' + line.strip())
                    score = 2
                else:
                    if self.debug == 1:
                        print ('Post hoc 1:\t' + line.strip())
                    score = max(score, 1)
        return score

    def __research_design(self, text_line, token_line):
        score = 0
        for line in text_line:
            if line.find('observation') >= 0 or line.find('experiment') >= 0:
                score = 1
                if self.debug == 1:
                    print ('Type of research design\t' + line.strip())
        return score

    def __data_collection_procedure(self, text_line, token_line):
        score = 0
        for line in text_line:
            if line.find('survey') >= 0 or line.find('interview') >= 0:
                if self.debug == 1:
                    print ('Data collection procedure:\t' + line.strip())
                score = 1
        return score

    def __country_of_data(self, text_line, token_line):
        score = 0
        for line in text_line:
            for country in self.__country_list:
                if line.find(country) >= 0:
                    score = 1
                    if self.debug == 1:
                        print ('Country:\t' + line.strip())
        return score

    def __sampling_method(self, text_line, token_line):
        score = 0
        for line in text_line:
            if line.find('purposeful sampl') >= 0 or line.find('snowball sampl') >= 0 \
                or line.find('convenience sampl') >= 0:
                score = 1
                if self.debug == 1:
                    print ('sampleing_method:\t' + line.strip())
        return score
        
    def __altered_items(self, text_line, token_line):
        score = 0
        in_method = False
        for tokens in token_line:
            if in_method is False:
                for tid in range(len(tokens)):
                    token = tokens[tid]
                    if token.startswith('method') and len(tokens) <= 3:
                        in_method = True
                        break
            #elif in_method is True:
            #    if len(tokens) <= 3:
            #        print ('out method')
            #        in_method = False

            if in_method is True:
                last_sent_end = -1
                begin_id = -1
                end_id = -1
                for tid in range(len(tokens)):
                    token = tokens[tid]
                    if token == 'adapted' or token == 'altered':
                        score = max(score, 1)
                        begin_id = last_sent_end
                        #print ('Altered item:\t1')
                    if token.endswith('.') and len(token) > 2 and not token[-2].isdigit():
                        if begin_id >= 0 and begin_id != last_sent_end:
                            end_id = tid
                        last_sent_end = tid
                if begin_id == -1:
                    continue
                for tid in range(begin_id + 1, end_id + 1):
                    token = tokens[tid]
                    if token == 'because' or token == 'for' or token == 'as' or \
                        (token == 'such' and tid < len(tokens) - 1 and tokens[tid + 1] == 'that'):
                        score = 2
                        #print ('Altered item:\t2')
        return score

    def __reversed_items(self, text_line, token_text):
        score = 0
        for line in text_line:
            if line.find('reverse coded') >= 0 or line.find('reverse-coded') >= 0 \
                or line.find('reverse scored') >= 0 or line.find('reverse-scored') >= 0:
                score = 1
                if self.debug == 1:
                    print ('Reversed items:\t' + line.strip())
        return score

    def __measurement_variability(self, text_line, token_line):
        score = 0
        for tokens in token_line:
            for tk in tokens:
                if tk.startswith('icc'):
                    score = 1
                    if self.debug == 1:
                        print ('Measurement variability:\t' + tk)
        return score

    def __range_restriction(self, text_line, token_line):
        score = 0
        for line in text_line:
            if line.find('range restriction') >= 0 or line.find('restriction of range') >= 0: 
               #\ or line.find('measurement error') >= 0 or line.find('error of measurement') >= 0:
                if line.find('correlation') >= 0:
                    score = 2
                else:
                    score = max(score, 1)
        return score

    def __analytic_method(self, text_line, token_line):
        score = 0
        for line in text_line:
            for met in self.__long_method:
                if line.find(met) >= 0:
                    if self.debug == 1:
                        print ('Analytic method long:\t' + line.strip())
                    score = 1
        for tokens in token_line:
            for token in tokens:
                for met in self.__short_method:
                    if met == token:
                        score = 1
                        if self.debug == 1:
                            print ('Analytic method short:\t' + line.strip())
        return score

    def __software(self, text_line, token_line):
        score = 0
        in_method = False
        for tokens in token_line:
            if in_method is False:
                if len(tokens) > 3:
                    continue
                for tid in range(len(tokens)):
                    token = tokens[tid]
                    if token.startswith('method') and len(tokens) <= 3:
                        in_method = True
                        break
            #elif in_method is True:
            #    if len(tokens) <= 3:
            #        print ('out method')
            #        in_method = False

            if in_method is True:
                last_sent_end = -1
                begin_id = -1
                end_id = -1
                for tid in range(len(tokens)):
                    token = tokens[tid]
                    for sw in self.__software_list:
                        if token == sw or token == sw + '.':
                            score = max(score, 1)
                            begin_id = last_sent_end + 1
                    if token.endswith('.') and len(token) > 2 and not token[-2].isdigit():
                        if begin_id >= 0:
                            end_id  = tid
                        last_sent_end = tid
                for tid in range(begin_id, end_id + 1):
                    token = tokens[tid]
                    if token.find('version') >= 0:
                        score = 2
                    elif is_number(token) and token.find('.') > 0:
                        score = 2
        return score

    def __outliers(self, text_line, token_line):
        score = 0
        for tokens in token_line:
            for tid in range(len(tokens)):
                token = tokens[tid]
                if token == 'outlier' or token == 'outliers':
                    if self.debug == 1:
                        print ('Outliers:\t' + ' '.join(tokens))
                    sent_cnt = 0
                    for j in range(tid + 1, len(tokens)):
                        if tokens[j].endswith('.'):
                            sent_cnt += 1
                    if sent_cnt > 2:
                        score = 2
                    else:
                        score = max(score, 1)
        return score
     
    def __sensitivity(self, text_line, token_line):
        score = 0
        for tokens in token_line:
            for tid in range(len(tokens)):
                token = tokens[tid]
                if token == 'sensitivity' and tid < len(tokens):
                    next_token = tokens[tid + 1]
                    if next_token.startswith('analys'):
                        if self.debug == 1:
                            print ('Missing data:\t' + ' '.join(tokens))
                        sent_cnt = 0
                        for j in range(tid + 2, len(tokens)):
                            if tokens[j].endwith('.'):
                                sent_cnt += 1
                        if sent_cnt > 2:
                            score = 2
                        else:
                            score = max(score, 1)
        return score            
     
    def __response_rate(self, text_line, token_line):
        score = 0
        for line in text_line:
            if line.find('response rate') >= 0:
                if self.debug == 1:
                    print ('Response rate')
                score = 1
        return score

    def __p_values(self, text_line, token_line):
        score = 0
        find_pvalue = False
        unequal_exist = False
        for tokens in token_line:
            for tid in range(len(tokens)):
                token = tokens[tid]
                if token == 'p' or token == '(p' or token == 'p-value':
                    find_pvalue = True
                    equal = 0
                    for k in range(tid + 1, min(tid + 3, len(tokens))):
                        if tokens[k] == '=':
                            equal = 1
                    if equal == 0:
                        unequal_exist = True
        if find_pvalue is True:
            if unequal_exist is False:
                return 2
            else:
                return 1
        return 0

    def __coefficients(self, text_line, token_line):
        work_zone = False
        find_token = False
        for tokens in token_line:
            if work_zone is False:
                #print (' '.join(tokens))
                if len(tokens) > 5:
                    continue
                for tid in range(len(tokens)):
                    token = tokens[tid]
                    if token.startswith('result'):
                        work_zone = True
                        break
            if work_zone is False:
                continue
            
            for tid in range(len(tokens)):
                token = tokens[tid]
                if token.endswith('standardized'):
                    find_token = True
                    break
            if find_token is True:
                break
        if find_token is True:
            return 1
        else:
            return 0
      

    def __effect_size(self, text_line, token_line):
        work_zone = False
        for tokens in token_line:
            if work_zone is False:
                if len(tokens) > 3:
                    continue
                for tid in range(len(tokens)):
                    token = tokens[tid]
                    if token == 'results' or token == 'discussion':
                        work_zone = True
                        break
            if work_zone is False:
                continue
            find_token = -1
            for tid in range(len(tokens)):
                token = tokens[tid]
                if token == 'r' or token == 'R' or token == 'r2' or token == 'ΔR2' or token == 'η2':
                    find_token = tid
                    if self.debug == 1:
                        print (token)
                        print (' '.join(tokens))
                    break
                if tid < len(tokens) - 1:
                    next_token = tokens[tid + 1]
                    if token == 'cohen\'s' and next_token == d:
                        find_token = tid
                        if self.debug == 1:
                            print (token)
                            print (' '.join(tokens))
                        break
                    elif token == 'odds' and next_token == 'ratio':
                        find_token = tid
                        if self.debug == 1:
                            print (token)
                            print (' '.join(tokens))
                        break
                    elif token == 'cramer\'s' and next_token == 'v':
                        find_token = tid
                        if self.debug == 1:
                            print (token)
                            print (' '.join(tokens))
                        break
            
            if find_token >= 0:
                for tid in range(find_token, len(tokens)):
                    token = tokens[tid].strip('.').strip('(').strip(')')
                    if is_number(token) is True:
                        return 4
                    if token.endswith('.'):
                        break
                return 2

        return 0


if __name__ == "__main__":
    file_name = sys.argv[1]
    ag = ArticleGrading()
    ag.grade_article(file_name)
